package com.nisum.BookInfo.controller;
import com.nisum.BookInfo.dto.BookDto;
import com.nisum.BookInfo.entity.Book;
import com.nisum.BookInfo.service.BookService;
import com.nisum.BookInfo.validator.CommonServiceValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;



@RestController
@RequestMapping("/books")

public class BookController {

    private static final Logger logger = LoggerFactory.getLogger(BookController.class);


    @Autowired
    public BookService bookService;
    @Autowired
    public CommonServiceValidator validator;

   /* @Autowired
    private ModelMapper modelMapper;


   @PostMapping("/createBook")
    public  Mono<Book> create(@RequestBody Book book){
       logger.info("CreateBook method called");
      return  bookService.create(book);


    }

   @GetMapping("/getAllBook")
    public Flux<Book> getAllBook(){
       logger.info("getAllBook method called");
       return bookService.getAllBooks();
    }

    @GetMapping("/getSingleBook/{bookId}")
    public Mono<Book> getSingleBook(@PathVariable Integer bookId){
        logger.info("getSingleBook method called");
       return  bookService.getBook(bookId);
    }

   @PutMapping("/updateBook/{bookId}")
    public Mono<Book> update(@RequestBody Book book,@PathVariable Integer bookId){
       logger.info("update method called");
       return bookService.updateBook(book,bookId);
    }

    @DeleteMapping("/deleteBook/{bookId}")
    public Mono<Void> delete(@PathVariable Integer bookId){
        logger.info("delete method called");
        return   bookService.deleteBook(bookId);
    }

*/


    // Create a new book
    @PostMapping("/createBook")
    public Mono<?> createBook(@RequestBody Mono<BookDto> bookDto) {
       /* return bookService.saveBook(bookDto)
                .map(rows -> "Book created successfully!");*/

        return bookDto.doOnNext(bookbookDto -> logger.info("validating incoming Book data: {}", bookDto))
                .flatMap(bookDto1 -> {

                    return validateBook(bookDto1)
                            .then(bookService.saveBook(bookDto1));
                })
                .map(ResponseEntity::ok)
                .doOnError(error -> logger.error("Error while adding Book: {}", error.getMessage()));
    }


    // Get all books
    @GetMapping("/getAllBook")
    public Flux<Book> getAllBooks() {
        //return bookService.getAllBooks();
        return bookService.getAllBooks()
                .doOnNext(employee -> logger.info("Processing Book data: {}", employee))
                .doOnError(error -> logger.error("Error fetching Book: {}", error.getMessage()));

    }

    // Get a book by IDBookService
    @GetMapping("/getBookById/{id}")
    public Mono<?> getBookById(@PathVariable Integer id) {
       // return bookService.getBookById(id);

        return validator.isBookIdValid(id)
                .flatMap(valid -> bookService.getBookById(id))
                .map(ResponseEntity::ok)
                .doOnError(error -> logger.error("Error fetching Book by ID: {}", error.getMessage()));

    }

    // Update a book by ID
    @PutMapping("/updateBook/{id}")
    public Mono<?> updateBook(@PathVariable Integer id, @RequestBody Mono<BookDto> bookDto) {
       /* return bookService.updateBook(id, bookDto)
                .map(rows -> "Book updated successfully!");*/

        return validator.isBookIdValid(id)
                .flatMap(valid -> bookDto
                        .flatMap(dto -> {

                            return validateBook(dto)
                                    .then(bookService.updateBook(id, dto));
                        })
                )
                .map(ResponseEntity::ok)
                .doOnError(error -> logger.error("Error updating Book: {}", error.getMessage()));
    }

    // Delete a book by ID
    @DeleteMapping("/deleteBookById/{id}")
    public Mono<?> deleteBook(@PathVariable Integer id) {
      /*  return bookService.deleteBook(id)
                .map(rows -> "Book deleted successfully!");*/

        return validator.isBookIdValid(id)
                .flatMap(valid -> bookService.deleteBook(id))
                .thenReturn(ResponseEntity.ok().build())
                .doOnError(error -> logger.error("Error deleting Book: {}", error.getMessage()));
    }



    private Mono<Void> validateBook(BookDto bookDto) {

        return validator.isBookIdValid(bookDto.getBookId())
                .then(validator.isBookNameValid(bookDto.getName()))
                .then();
    }

}


