package com.nisum.BookInfo.dao;


import com.nisum.BookInfo.entity.Book;
import com.nisum.BookInfo.exception.BookNotFoundException;
import com.nisum.BookInfo.util.AppErrorCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.r2dbc.core.DatabaseClient;
import org.springframework.stereotype.Repository;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Repository
public class BookDao {
    @Autowired
    private DatabaseClient databaseClient;




    public Mono<Book> saveBook(Book book) {
        return databaseClient
                .sql("INSERT INTO book_details (book_id, book_name,book_desc,book_publiser,book_author) VALUES (:bookId, :name,:desc,:publiser,:author)")
                .bind("bookId", book.getBookId())
                .bind("name", book.getName())
                .bind("desc", book.getDesc())
                .bind("publiser", book.getPubliser())
                .bind("author", book.getAuthor())
                .fetch()
                .rowsUpdated().flatMap(rows ->
                        Mono.justOrEmpty(book) // empty if book is null)
                                .switchIfEmpty(Mono.error(new BookNotFoundException(AppErrorCode.BOOK_NOT_FOUND, "Data Invalid"))) // If book is null, throw error
                                .onErrorResume(BookNotFoundException.class, ex -> {
                                    // Handle specific error if needed
                                    return Mono.error(ex); // Propagate the original error if required
                                })
                );

        /*flatMap(rows ->
                {
                    if (rows > 0) {
                        return Mono.just(book).onErrorResume();  //Returning the saved employee
                    } else {
                        return Mono.error(new BookNotFoundException(AppErrorCode.BOOK_NOT_FOUND,"Data Invalid") {
                        });

                    }
                });*/
    }


    public Flux<Book> getAllBooks() {
        return databaseClient
                .sql("SELECT book_id, book_name, book_desc, book_publiser,book_author FROM book_details")
                .map((row, metadata) -> new Book(
                        row.get("book_id", Integer.class),
                        row.get("book_name", String.class),
                        row.get("book_desc", String.class),
                        row.get("book_publiser", String.class),
                        row.get("book_author", String.class)
                ))
                .all();      // Returns Flux<Book> for all books
    }

    public Mono<Book> getBookById(Integer id) {
        return databaseClient
                .sql("SELECT book_id, book_name, book_desc, book_publiser,book_author FROM book_details WHERE book_id = :book_id")
                .bind("book_id", id)
                .map((row, metadata) -> new Book(
                        row.get("book_id", Integer.class),
                        row.get("book_name", String.class),
                        row.get("book_desc", String.class),
                        row.get("book_publiser", String.class),
                        row.get("book_author", String.class)

                ))
                .one(); // Returns Mono<Book> for a single book or empty if not found
    }

    // Update a book
    public Mono<Book> updateBook(Integer bookId, Book book) {
        return databaseClient
                .sql("UPDATE book_details SET  book_author = :author, book_name = :name WHERE book_id = :id")
                .bind("id", bookId)
                .bind("name", book.getAuthor())
                //.bind("desc", book.getDesc())
                //.bind("publiser", book.getPubliser())
                .bind("author", book.getAuthor())

                .fetch()
                .rowsUpdated().flatMap(rows ->
                        Mono.justOrEmpty(rows) // Wrap rows in Mono
                                .filter(r -> r > 0) // Only continue if rows > 0, else it will trigger error handling
                                .switchIfEmpty(Mono.error(new BookNotFoundException(AppErrorCode.BOOK_UPDATE_FAILED, "UPDATE OF BOOK NOT POSSIBLE"))) // If rows <= 0, trigger error
                                .flatMap(r -> Mono.just(book)) // If rows > 0, return the book
                );




                /*flatMap(rows ->
                {
                    if (rows > 0) {
                        return Mono.just(book);  //Returning the saved employee
                    } else {
                        return Mono.error(new BookNotFoundException(AppErrorCode.BOOK_UPDATE_FAILED,"UPDATE OF BOOK NOT POSSIBLE") {
                                          }
                        );
                    }


                });*/
    }

    public Mono<Void> deleteBook(Integer id) {
        return databaseClient.sql("DELETE FROM book_details WHERE book_id = :id")
                .bind("id", id)
                .fetch()
                .rowsUpdated().flatMap(rows ->
                        Mono.justOrEmpty(rows) // Wrap rows in Mono
                                .filter(r -> r > 0) // Continue if rows > 0, else proceed to error handling
                                .switchIfEmpty(Mono.error(new BookNotFoundException(AppErrorCode.BOOK_DELETION_FAILED, "Invalid Book data"))) // Trigger error if rows <= 0
                                .then(Mono.empty()) // If rows > 0, return Mono.empty()
                );



                /*flatMap(rows ->
                {

                    if (rows > 0) {
                        return Mono.empty();


                    } else {
                        return Mono.error(new BookNotFoundException(AppErrorCode.BOOK_DELETION_FAILED,"Invalid Book data") {
                                          }
                        );
                    }


                });*/
    }

}
