package com.nisum.BookInfo.validator;

import com.nisum.BookInfo.entity.Book;
import com.nisum.BookInfo.exception.BookNotFoundException;
import com.nisum.BookInfo.exception.BookValidationException;
import com.nisum.BookInfo.util.AppErrorCode;
import org.springframework.stereotype.Component;
import reactor.core.publisher.Mono;
@Component
public class CommonServiceValidator {



//bookId is invalid
    public Mono<Boolean> isBookIdValid(Integer id) {
        if (id != null) {
            return Mono.just(true);
        }
        return Mono.error(new BookValidationException(AppErrorCode.INVALID_BOOK_DATA, "Book ID cannot be null"));
    }


    // Validate if Book name is not null or empty
    public Mono<Boolean> isBookNameValid(String name) {
        if (name != null && !name.trim().isEmpty()) {
            return Mono.just(true);
        }
        return Mono.error(new BookValidationException(AppErrorCode.INVALID_BOOK_DATA ,"Book name cannot be empty or null"));
    }

    public Mono<Boolean> isBookPubliserValid(String publiserName) {
        if (publiserName != null && !publiserName.trim().isEmpty()) {
            return Mono.just(true);
        }
        return Mono.error(new BookNotFoundException(AppErrorCode.BOOK_CREATION_FAILED ,"Not an valid publiser"));
    }
    public Mono<Boolean> isBookObjectValid(Book book) {
        if (book == null) {
            return Mono.error(new BookNotFoundException(AppErrorCode.BOOK_NOT_FOUND, "Book object cannot be null"));
        }

        return isBookIdValid(book.getBookId())
                .flatMap(valid -> isBookNameValid(book.getName()));

    }
}
