package com.nisum.BookInfo.dao;

import com.nisum.BookInfo.dto.BookDto;
import com.nisum.BookInfo.entity.Book;
import com.nisum.BookInfo.service.BookService;
import com.nisum.BookInfo.service.BookServiceImpl;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.r2dbc.core.DatabaseClient;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;

import static org.mockito.Mockito.*;

@SpringBootTest
class BookDaoTest {

  @Mock
   // private DatabaseClient databaseClient;

    @InjectMocks
    private BookServiceImpl bookService;



    private Book book;

    @BeforeEach
   /* public void setUp() {

      //  book = new Book(1, "Java Book", "A test description", "Test Publisher", "Test Author");
      //  BookDto bookDto = new BookDto(1, "C++", "This is c++", "C publiser", "Test Author");
       // Book book = new Book(1, "C++", "This is c++", "C publiser", "Test Author");
    }*/

    @Test
    public void testSaveBook_Success() {
        BookDto bookDto = new BookDto(1, "C++", "This is c++", "C publiser", "Test Author");
        Book book = new Book(1, "C++", "This is c++", "C publiser", "Test Author");

        DatabaseClient.GenericExecuteSpec executeSpec = mock(DatabaseClient.GenericExecuteSpec.class);

        when(databaseClient.sql("INSERT INTO book_details (book_id, book_name, book_desc, book_publiser, book_author) VALUES (:bookId, :name, :desc, :publiser, :author)"))
                .thenReturn(executeSpec);
        when(executeSpec.bind(1, bookDto.getBookId())).thenReturn(executeSpec);
        when(executeSpec.bind("C++", bookDto.getName())).thenReturn(executeSpec);
        when(executeSpec.bind("This is c++", bookDto.getDesc())).thenReturn(executeSpec);
        when(executeSpec.bind("C publiser", bookDto.getPubliser())).thenReturn(executeSpec);
        when(executeSpec.bind("Test Author", bookDto.getAuthor())).thenReturn(executeSpec);


        // Call the saveBook method
        Mono<Book> result = bookService.saveBook(bookDto);


        StepVerifier.create(result)
                .expectNext(book)  // Expect the book to be returned after saving
                .verifyComplete();  // The Mono should complete successfully
    }

    @Mock
    private DatabaseClient databaseClient; // Mocked database client

    @InjectMocks
    private BookDao bookDao;  // Class under test

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);  // Initialize mocks
    }

    @Test
    void saveBook_shouldReturnBook_whenInsertionIsSuccessful() {
        // Arrange
        Book book = new Book(1, "Test Book", "A test description", "Test Publisher", "Test Author");

        // Mock the databaseClient behavior
        DatabaseClient.GenericExecuteSpec executeSpec = mock(DatabaseClient.GenericExecuteSpec.class);
        when(databaseClient.sql(anyString())).thenReturn(executeSpec);
        when(executeSpec.bind(anyString(), any())).thenReturn(executeSpec);
       // when(executeSpec.fetch()).thenReturn(mock(DatabaseClient.RowsUpdatedSpec.class));
       // when(executeSpec.rowsUpdated()).thenReturn(Mono.just(1L)); // Simulate a successful insert (1 row updated)

        // Act & Assert
        StepVerifier.create(bookDao.saveBook(book))
                .expectNext(book)  // Expect the saved book to be returned
                .verifyComplete(); // Ensure the Mono completes successfully
    }

    @Test
    void saveBook_shouldReturnError_whenInsertionFails() {
        // Arrange
        Book book = new Book(123, "Test Book", "A test description", "Test Publisher", "Test Author");

        // Mock the databaseClient behavior
        DatabaseClient.GenericExecuteSpec executeSpec = mock(DatabaseClient.GenericExecuteSpec.class);
        when(databaseClient.sql(anyString())).thenReturn(executeSpec);
        when(executeSpec.bind(anyString(), any())).thenReturn(executeSpec);
        //when(executeSpec.fetch()).thenReturn(mock(DatabaseClient.RowsUpdatedSpec.class));
       // when(executeSpec.rowsUpdated()).thenReturn(Mono.just(0L)); // Simulate a failure (0 rows updated)

        // Act & Assert
        //StepVerifier.create(bookDao.saveBook(book))
             //   .expectError(BookInsertException.class) // Expect the custom exception
             //   .verify(); // Ensure the Mono emits an error
    }
}








